@@ -1,13 +1,14 @@
-from typing import Any, Callable, List, Optional, TYPE_CHECKING, Type, Union
+from typing import Any, Generator, List, Optional, TYPE_CHECKING, Type, Union
 
 import sqlalchemy
+from sqlalchemy import UniqueConstraint
 
 import ormar  # noqa I101
 from ormar.exceptions import RelationshipInstanceError
 from ormar.fields.base import BaseField
 
 if TYPE_CHECKING:  # pragma no cover
-    from ormar.models import Model
+    from ormar.models import Model, NewBaseModel
 
 
 def create_dummy_instance(fk: Type["Model"], pk: Any = None) -> "Model":
@@ -22,6 +23,10 @@ def create_dummy_instance(fk: Type["Model"], pk: Any = None) -> "Model":
     return fk(**init_dict)
 
 
+class UniqueColumns(UniqueConstraint):
+    pass
+
+
 def ForeignKey(  # noqa CFQ002
     to: Type["Model"],
     *,
@@ -32,7 +37,7 @@ def ForeignKey(  # noqa CFQ002
     virtual: bool = False,
     onupdate: str = None,
     ondelete: str = None,
-) -> Type[object]:
+) -> Type["ForeignKeyField"]:
     fk_string = to.Meta.tablename + "." + to.Meta.pkname
     to_field = to.__fields__[to.Meta.pkname]
     namespace = dict(
@@ -65,7 +70,7 @@ class ForeignKeyField(BaseField):
     virtual: bool
 
     @classmethod
-    def __get_validators__(cls) -> Callable:
+    def __get_validators__(cls) -> Generator:
         yield cls.validate
 
     @classmethod
@@ -75,8 +80,11 @@ class ForeignKeyField(BaseField):
     @classmethod
     def _extract_model_from_sequence(
         cls, value: List, child: "Model", to_register: bool
-    ) -> Union["Model", List["Model"]]:
-        return [cls.expand_relationship(val, child, to_register) for val in value]
+    ) -> List["Model"]:
+        return [
+            cls.expand_relationship(val, child, to_register)  # type: ignore
+            for val in value
+        ]
 
     @classmethod
     def _register_existing_model(
@@ -120,7 +128,7 @@ class ForeignKeyField(BaseField):
 
     @classmethod
     def expand_relationship(
-        cls, value: Any, child: "Model", to_register: bool = True
+        cls, value: Any, child: Union["Model", "NewBaseModel"], to_register: bool = True
     ) -> Optional[Union["Model", List["Model"]]]:
         if value is None:
             return None if not cls.virtual else []
@@ -131,7 +139,7 @@ class ForeignKeyField(BaseField):
             "list": cls._extract_model_from_sequence,
         }
 
-        model = constructors.get(
+        model = constructors.get(  # type: ignore
             value.__class__.__name__, cls._construct_model_from_pk
         )(value, child, to_register)
         return model
