@@ -1,7 +1,7 @@
 import string
 import uuid
 from random import choices
-from typing import List
+from typing import Dict, List
 
 import sqlalchemy
 from sqlalchemy import text
@@ -14,13 +14,21 @@ def get_table_alias() -> str:
 
 class AliasManager:
     def __init__(self) -> None:
-        self._aliases = dict()
+        self._aliases: Dict[str, str] = dict()
 
     @staticmethod
-    def prefixed_columns(alias: str, table: sqlalchemy.Table) -> List[text]:
+    def prefixed_columns(
+        alias: str, table: sqlalchemy.Table, fields: List = None
+    ) -> List[text]:
+        alias = f"{alias}_" if alias else ""
+        all_columns = (
+            table.columns
+            if not fields
+            else [col for col in table.columns if col.name in fields]
+        )
         return [
-            text(f"{alias}_{table.name}.{column.name} as {alias}_{column.name}")
-            for column in table.columns
+            text(f"{alias}{table.name}.{column.name} as {alias}{column.name}")
+            for column in all_columns
         ]
 
     @staticmethod
